import requests, json, re
from bs4 import BeautifulSoup
client = requests.session()

url_index = 'https://espa.cr.usgs.gov/index/'
url_login = 'https://ers.cr.usgs.gov/login/'
url_order = 'https://espa.cr.usgs.gov/ordering/status/'
url_download = 'https://espa.cr.usgs.gov/ordering/order-status/'

def login(username, pwd):
    rst = client.get(url_login)
    cont = rst.content.decode('utf-8')
    loc = cont.index('==')
    token = cont[loc-6:loc+2]
    data = {'username':username,'password':pwd,'csrf_token':token}
    rst = client.post(url_login, data)
    if rst.status_code!=502: print('Error:login')
    
def list_order():
    rst = client.get(url_order)
    cont = rst.content.decode('utf-8')
    p = re.compile('>espa-kongwp@radi.ac.cn-\d+-\d+-\d+<')
    return [i[1:-1] for i in p.findall(cont)]

def list_file(orderid):
    rst = client.get(url_download+orderid)
    cont = rst.content.decode('utf-8')
    patten = re.compile('https://edclpdsftp.*tar.gz')
    return patten.findall(cont)

def download(files):
    for imgurl in files:
        print('downloading', imgurl.split('/')[-1], '...', end='')
        try:
            rst = client.get(imgurl)
            f = open('download/'+imgurl.split('/')[-1], 'wb')
            f.write(rst.content)
            f.close()
            print('ok')
        except: print('failed!')
    print('completed!')
    
if __name__ == '__main__':
    login('weiping_RADI', '123ca#456') # 登录
    orders = list_order() # 列举订单
    files = list_file(orders[0]) # 列举某个订单中的文件
    download(files) # 下载文件



from scipy.ndimage import distance_transform_edt as edt
import numpy as np

def degap(img, msk, r=0):
	dis, indices = edt(msk, return_indices=True)
	if r!=0: msk = msk & (dis<r)
	if isinstance(img, list): imgs = img
	else: imgs = [img] if img.ndim==2 else img
	rc = [i[msk] for i in indices]
	for im in imgs: im[msk] = im[tuple(rc)]
	return img


from shapely.affinity import affine_transform
from shapely.geometry import GeometryCollection
from PIL import Image, ImageDraw, ImageFont
from shapely.geometry import Polygon, LineString
import numpy as np, pandas as pd, pyproj
from numba import jit

def draw_polygon(raster, shape, color, width):
    shape = shape.to_crs(raster.crs)
    img = Image.fromarray(raster)
    draw = ImageDraw.Draw(img)
    geoms = shape['geometry'].affine_transform(raster.imat1)
    if isinstance(color, np.ndarray): colors = color.tolist()
    elif isinstance(color, pd.Series): colors = color.to_list()
    elif isinstance(color, str): colors = shape[color].to_list()
    else: colors = [color] * len(geoms)
    for g, c in zip(geoms, colors):
        gs = g #affine_transform(g, m)
        if isinstance(gs, Polygon): gs = [gs]
        if isinstance(gs, LineString): gs = [gs]
        for g in gs:
            pts = np.array(g.exterior.xy).T.astype(np.int).reshape((-1,2))
            pts = [tuple(i) for i in pts]
            if width==0: draw.polygon(pts, c)
            if width!=0: draw.line(pts, c, width, joint='curve')
    raster[:] = np.array(img)

def draw_line(raster, shape, color, width):
    shape = shape.to_crs(raster.crs)
    m = raster.mat
    m = [1/m[0,1], 0, 0, -1/m[0,1], -m[0,0]/m[0,1], m[1,0]/m[0,1]]
    img = Image.fromarray(raster)
    draw = ImageDraw.Draw(img)
    geoms = shape['geometry']
    if isinstance(color, pd.Series): colors = color.to_list()
    elif isinstance(color, str): colors = shape[color].to_list()
    else: colors = [color] * len(geoms)
    for g, c in zip(geoms, colors):
        gs = affine_transform(g, m)
        if isinstance(gs, LineString): gs = [gs]
        for g in gs:
            pts = np.array(g.xy).T.astype(np.int).reshape((-1,2))
            pts = [tuple(i) for i in pts]
            draw.line(pts, c, width, joint='curve')
    raster[:] = np.array(img)

def draw_mask(img, msk):
    weight = msk[:,:,3]/255
    np.multiply(img.T, 1-weight.T, out=img.T, casting='unsafe')
    np.add(img.T, msk[:,:,:3].T*weight.T, out=img.T, casting='unsafe')

@jit
def draw_mask(img, msk):
    h, w = img.shape[:2]
    for r in range(h):
        for c in range(w):
            wg = msk[r,c,3]/255.0
            img[r,c] = img[r,c]*(1-wg) + msk[r,c,:3]*wg
    return img

def draw_unit(raster, x, y, w, h, ft, color, unit, lw, anc='left'):
    y = y-h
    img = Image.fromarray(raster)
    d = ImageDraw.Draw(img)
    def f(x, dir=0):
        v = raster.shape[1-dir]
        if abs(x)>1: x = int(x)
        else: x = int(v * x)
        return x if x>=0 else v+x
    step = int(round(f(w,0)*raster.mat[0,1]/10000))
    if step>100:
        n = 10**len(str(step))/10
        step = int(np.round(step/n)*n)
    cell = int(step * 1000 / raster.mat[0,1])
    offsetx = 0 if anc=='left' else -cell *10
    for s, e, c in zip((0,1,3,6), (1,3,6,10), (1,0,1,0)):
        d.rectangle([f(x)+s*cell+offsetx, f(y,1), 
                     f(x)+e*cell+offsetx, f(y,1)+f(h)],
                     fill=[color, None][c], outline=color, width=lw)
    font = ImageFont.truetype(*ft)
    for s in (0,1,3,6,10):
        w, h = d.textsize(str(step*(s))+['',' '+unit][s==10], font)
        d.text((f(x)+s*cell-w//2+offsetx, f(y,1)-ft[1]*1.2),
               str(step*(s))+['',' '+unit][s==10], font=font, fill=color, align='center')
    raster[:] = np.array(img)

def draw_N(raster, x, y, ft, lw, h, color):
    img = Image.fromarray(raster)
    d = ImageDraw.Draw(img)
    def f(x, dir=0):
        v = raster.shape[1-dir]
        if abs(x)>1: x = int(x)
        else: x = int(v * x)
        return x if x>=0 else v+x
    x, y = f(x), f(y, 1)
    d.polygon([x, y, x, y+h, x-h/2, y+h*5//3], color)
    d.line([x, y, x-h/2, y+h*5//3, x, y+h, x+h/2, y+h*5//3, x, y], color, lw)
    font = ImageFont.truetype(*ft)
    w,h = d.textsize('N', font)
    x, y = x-w//2, y-h*1.2
    d.text((x, y), 'N', font=font, fill=color)
    raster[:] = np.array(img)

def draw_text(raster, txt, xs, ys, color, ft, anc='lt', align='left'):
    img = Image.fromarray(raster)
    d = ImageDraw.Draw(img)
    if isinstance(txt, str): txt = [txt]
    if not hasattr(xs, '__len__'): xs = [xs]
    if not hasattr(ys, '__len__'): ys = [ys]
    def f(x, dir=0):
        v = raster.shape[1-dir]
        if abs(x)>1: x = int(x)
        else: x = int(v * x)
        return x if x>=0 else v+x
    font = ImageFont.truetype(*ft)
    for t, x,y in zip(txt, xs, ys):
        x, y = f(x), f(y,1)
        w,h = d.textsize(t, font)
        if anc=='ct': x, y = x-w//2, y-h*3/5
        if anc=='lb': y = y-h
        if anc=='lt': x, y = x, y
        if anc=='rb': x, y = x-w, y-h
        if anc=='rt': x = x-w
        d.text((x, y), t, font=font, fill=color, align=align, spacing=ft[1]*0.3)
    raster[:] = np.array(img)

def draw_lab(raster, shp, name, color, font, anc):
    gs = shp['geometry'].centroid
    m = raster.mat
    m = [1/m[0,1], 0, 0, -1/m[0,1], -m[0,0]/m[0,1], m[1,0]/m[0,1]]
    pos = [(int(p.x), int(p.y)) for p in [affine_transform(i, m) for i in gs]]
    return draw_text(raster, shp[name], *list(zip(*pos)), color, font, anc)

def draw_style(raster, x, y, body, mar, recsize, font, color, box):
    body = body[::-1]
    img = Image.fromarray(raster)
    d = ImageDraw.Draw(img)
    tcolor = color
    def f(x, dir=0):
        v = raster.shape[1-dir]
        if abs(x)>1: x = int(x)
        else: x = int(v * x)
        return x if x>=0 else v+x
    cury, maxx, (w, h, lw) = f(y,1)-mar[1], 0, recsize
    font = ImageFont.truetype(*font[:2])
    for i in range(len(body)):
        item = body[i]
        if item[0]=='line':
            _, c, t = item
            d.line([f(x)+mar[0], cury-f(h,1)//2, f(x)+mar[0]+f(w), 
                cury-f(h,1)//2], fill=c, width=lw)
            tw, th = d.textsize(t, font)
            maxx = max(maxx, f(x)+mar[0]*3+f(w)+tw)
            d.text([f(x)+mar[0]*2+f(w), cury-f(h,1)//2-th*3/5], t, 
                font=font, fill=tcolor)
            cury -= mar[1]+f(h,1)
        elif item[0]=='rect':
            _, c, t = item
            d.rectangle([f(x)+mar[0], cury, f(x)+mar[0]+f(w), cury-f(h,1)], 
                fill=c, outline=[None, tcolor][lw>0], width=lw)
            tw, th = d.textsize(t, font)
            maxx = max(maxx, f(x)+mar[0]*3+f(w)+tw)
            d.text([f(x)+mar[0]*2+f(w), cury-f(h,1)//2-th*3/5], t, 
                font=font, fill=tcolor)
            cury -= mar[1]+f(h,1)
        elif item[0]=='circle':
            _, c, t = item
            d.ellipse([f(x)+mar[0]+f(w)//2-f(h,1)//5, cury-f(h,1)//2-f(h,1)//5,
                       f(x)+mar[0]+f(w)//2+f(h,1)//5, cury-f(h,1)//2+f(h,1)//5],
                       fill=c, outline=[None, tcolor][lw>0], width=lw)
            tw, th = d.textsize(t, font)
            maxx = max(maxx, f(x)+mar[0]*3+f(w)+tw)
            d.text([f(x)+mar[0]*2+f(w), cury-f(h,1)//2-th*3/5], t, 
                font=font, fill=tcolor)
            cury -= mar[1]+f(h,1)
        elif item[0]=='blank':
            cury -= item[1]
        else:
            print(item)
            t, tf, ts = item
            tf = ImageFont.truetype(tf, ts)
            tw, th = d.textsize(t, tf)
            d.text([f(x)+mar[0]*2, cury-f(h,1)//2-th*3/5], 
                t, font=tf, fill=tcolor)
            cury -= mar[1]+f(h,1)
    if box>0:
        d.rectangle([f(x), f(y,1), maxx, cury], outline=tcolor, width=box)
    raster[:] = np.array(img)

def draw_ruler(raster, left, top, right, bot, step, crs, font, color, w, dh): 
    img = Image.fromarray(raster)
    d = ImageDraw.Draw(img)
    def f(x, dir=0):
        v = raster.shape[1-dir]
        if abs(x)>1: x = int(x)
        else: x = int(v * x)
        return x if x>=0 else v+x
    left, top, right, bot = f(left), f(top,1), f(right), f(bot,1)
    d.rectangle([left, top, right, bot], outline=color, width=w)
    pts = np.array([(left,top),(right,top),(right,bot),(left,bot)])
    prj1, prj2 = pyproj.CRS(raster.crs), pyproj.CRS(crs)
    ct = pyproj.Transformer.from_crs(prj1, prj2, always_xy=True)
    pts = np.dot(raster.mat[:,1:], pts.T) + raster.mat[:,:1]
    xs, ys = ct.transform(*pts)

    a, b = int(xs[0]//step), int(xs[1]//step)
    tab = [i*step for i in range(a+1, b+1)]
    nps = ct.transform(tab, np.linspace(ys[0], ys[1], len(tab)), direction='INVERSE')
    nps = np.dot(np.linalg.inv(raster.mat[:,1:]), nps - raster.mat[:,:1])
    font = ImageFont.truetype(*font)
    for x,e in zip(nps[0],tab):
        d.line([int(x), top, int(x), top-dh], color, w)
        tw, th = d.textsize('%d°E'%e, font)
        d.text((int(x-tw//2), top-dh-th*1.2), '%d°E'%e, font=font, fill=color)

    a, b = int(xs[3]//step), int(xs[2]//step)
    tab = [i*step for i in range(a+1, b+1)]
    nps = ct.transform(tab, np.linspace(ys[3], ys[2], len(tab)), direction='INVERSE')
    nps = np.dot(np.linalg.inv(raster.mat[:,1:]), nps - raster.mat[:,:1])
    for x,e in zip(nps[0],tab):
        d.line([int(x), bot, int(x), bot+dh], color, w)
        tw, th = d.textsize('%d°E'%e, font)
        d.text((int(x-tw//2), bot+dh+th*0.0), '%d°E'%e, font=font, fill=color)

    a, b = int(ys[3]//step), int(ys[0]//step)
    tab = [i*step for i in range(a+1, b+1)]
    nps = ct.transform(np.linspace(xs[3], xs[0], len(tab)), tab, direction='INVERSE')
    nps = np.dot(np.linalg.inv(raster.mat[:,1:]), nps - raster.mat[:,:1])
    for y,n in zip(nps[1],tab):
        d.line([left, int(y), left-dh, int(y)], color, w)
        tw, th = d.textsize('%d°N'%n, font)
        d.text((left-dh-tw-th*0.2, int(y-th*3/5)), '%d°N'%n, font=font, fill=color)

    a, b = int(ys[2]//step), int(ys[1]//step)
    tab = [i*step for i in range(a+1, b+1)]
    nps = ct.transform(np.linspace(xs[2], xs[1], len(tab)), tab, direction='INVERSE')
    nps = np.dot(np.linalg.inv(raster.mat[:,1:]), nps - raster.mat[:,:1])
    for y,n in zip(nps[1],tab):
        d.line([right, int(y), right+dh, int(y)], color, w)
        tw, th = d.textsize('%d°N'%n, font)
        d.text((right+dh+th*0.2, int(y-th*3/5)), '%d°N'%n, font=font, fill=color)
    raster[:] = np.array(img)

def draw_bound(raster, left, top, right, bot, c, lw, clear=None):
    def f(x, dir=0):
        v = raster.shape[1-dir]
        if abs(x)>1: x = int(x)
        else: x = int(v * x)
        return x if x>=0 else v+x
    left, top, right, bot = f(left), f(top,1), f(right), f(bot,1)
    if not clear is None:
        raster[:top] = clear
        raster[bot:] = clear
        raster[:,:left] = clear
        raster[:,right:] = clear
    img = Image.fromarray(raster)
    d = ImageDraw.Draw(img)
    d.rectangle([left, top, right, bot], outline=c, width=lw)
    raster[:] = np.array(img)


import urllib.request as request
import os, datetime, json

url = 'https://ladsweb.modaps.eosdis.nasa.gov/archive/allData'

def get_lev_pro_year(level, product, year):
    ref = url + '/%s/%s/%s.json'%(level, product, year)
    req = request.Request(ref)
    response = request.urlopen(req)
    return json.loads(response.read().decode('utf-8'))

def get_list(level, product, year, day):
    ref = url + '/%s/%s/%s/%s.json'%(level, product, year, day)
    req = request.Request(ref)
    response = request.urlopen(req)
    return json.loads(response.read().decode('utf-8'))

def get_all(level, product, areas, year, days = (0,365)):
    print('searching... %s level %s %s'%(product, level, year))
    if isinstance(days, list): days = ['%.3d'%i for i in days]
    else:
        buf = [i['name'] for i in get_lev_pro_year(level, product, year)]
        days = [i for i in buf if int(i)>=days[0] and int(i)<=days[1]]
    files = []
    for day in days:
        print('\t%s day ...'%day, end=' ')
        lst = [i['name'] for i in get_list(level, product, year, day)]
        files.extend([(level, product, year, day), i] for i in lst)
        print('done')
    print()
    files = [i for i in files if sum([a in i[1] for a in areas])]
    return files

def download_one(pre, name, des):
    ref = url + '/%s/%s/%s/%s/%s'%(pre+(name,))
    req = request.Request(ref)
    response = request.urlopen(req)
    html = response.read()
    f = open('%s/%s'%(des,name), 'wb')
    f.write(html)
    f.close()

def search(product, level, areas, terms):
    files = []
    for pro in product:
        for year, days in terms:
            files.extend(get_all(level, pro, areas, year, days))
    print('%s new files found!'%len(files))
    return files

def download(files, des):
    succ = 0
    for pre, name in files:
        try:
            print('\t', name, end=' ... ')
            download_one(pre, name, des)
            print('done')
            succ += 1
        except:
            print('failed!')
    print('download completed, succ %s，failed %s'%(succ, len(files)-succ))


    
if __name__ == '__main__':
    files = search(['MOD09Q1', 'MOD11A2'], level=6, areas=['h25v05', 'h27v05', 'h28v05'], terms=[(2019, (0,30))])
    download(files, '')



from osgeo import gdal, osr
import pyproj
import geopandas as gpd
from glob import glob
import numpy as np
from numpy.linalg import inv
import geonumpy as gnp
import geonumpy.util as gutil
import geonumpy.io as gio
from math import ceil, floor
from scipy.ndimage import map_coordinates

def mp2pm(boxs, m1, prj1, prj2, m2, t1 = lambda x:x, t2 = lambda x:x):
    p1 = osr.SpatialReference()
    p1.ImportFromProj4(pyproj.CRS(prj1).to_proj4())
    p2 = osr.SpatialReference()
    p2.ImportFromProj4(pyproj.CRS(prj2).to_proj4())
    box = t1(np.dot(m1[:,1:], np.array(boxs).T) + m1[:,:1])
    ct = osr.CoordinateTransformation(p1, p2)
    box = t2(np.array(ct.TransformPoints(box.T)).T)
    return np.dot(inv(m2[:,1:]), box[:2]-m2[:,:1]).T
    '''
    prj1 = pyproj.CRS(wkt1)
    prj2 = pyproj.CRS(wkt2)
    ct = pyproj.Transformer.from_crs(prj1, prj2)
    print(ct.transform(116, 39))
    '''
    
def match_one(raster, out, chan='all', step=10, order=1):
    if chan=='all': chan = list(range(raster.channels()))
    if isinstance(chan, int): chan = [chan]
    if isinstance(out, tuple):
        out = gnp.frombox(*out, len(chan), raster.dtype)
    imgd, prjd, md = out, out.crs, out.mat
    img, prjs, ms = raster, raster.crs, raster.mat
    hs, ws = img.shape[:2]
    xx = np.linspace(0,ws,100).reshape((-1,1))*[1,0]
    yy = np.linspace(0,hs,100).reshape((-1,1))*[0,1]
    xy = np.vstack((xx, xx+[0,hs], yy, yy+[ws,0]))
    xy = mp2pm(xy, ms, prjs, prjd, md)#.astype(np.int)
    
    (left, top), (right, bot) = xy.min(axis=0), xy.max(axis=0)
    left, right = np.clip((left, right), 0, imgd.shape[1])
    top, bot = np.clip((top, bot), 0, imgd.shape[0])
    hb, wb = bot-top, right-left # 像素数目
    nh, nw = int(hb//step+1), int(wb//step+1)
    xy = np.mgrid[top:bot:nh*1j, left:right:nw*1j].reshape((2,-1)).T

    xs, ys = mp2pm(xy[:,::-1], md, prjd, prjs, ms).T

    xs.shape = ys.shape = (nh, nw)#block.shape

    intleft, intright = floor(left), ceil(right)
    inttop, intbot = floor(top), ceil(bot)
    rc = np.mgrid[inttop:intbot, intleft:intright].reshape((2,-1))
    frc = rc.astype(np.float32) + 0.5
    frc[0] = (frc[0]-top)/(bot-top)*(nh-1)
    frc[1] = (frc[1]-left)/(right-left)*(nw-1)
    xs = map_coordinates(xs, frc, order=1)
    ys = map_coordinates(ys, frc, order=1)
    for i in chan:
        #range(img.channels()):
        rcs, dcs = img.channels(i), imgd.channels(i)
        vs = map_coordinates(rcs, np.array([ys, xs])-0.5, order=order)#prefilter=False)
        vs[np.isnan(vs)] = 0
        dcs[tuple(rc)] = np.maximum(dcs[tuple(rc)], vs)   
    return imgd

def match_multi(rasters, out, chan='all', step=10, order=1):
    if chan=='all': chan = list(range(rasters[0].channels()))
    if isinstance(chan, int): chan = [chan]
    if isinstance(out, tuple):
        out = gnp.frombox(*out, len(chan), rasters[0].dtype)
    for raster in rasters: match_one(raster, out, chan, step, order)
    return out

def build_index(fs):
    boxes, bcrs = [], None
    for i in fs:
        isfile = isinstance(i, str)
        if isfile: shp, crs, m, chans = gio.read_raster_box(i)
        else: (shp, crs, m), chans = i.getbox(), i.channels()
        if bcrs is None: bcrs = crs
        box = gutil.box2shp(shp, crs, m).to_crs(bcrs)
        boxes.append([box[0], shp, m, chans, ['memory', i][isfile]])
    columns = ['geometry', 'shape', 'mat', 'channels', 'path']
    return gpd.GeoDataFrame(boxes, columns=columns, crs=bcrs)

def match_idx(idx, out, chan='all', step=10, order=1):
    if chan=='all': chan = list(range(len(idx['channels'][0])))
    if isinstance(chan, int): chan = [chan]
    if isinstance(out, tuple): shape, crs, mat = out
    else: shape, crs, mat = out.shape, out.crs, out.mat
    idx = idx.to_crs(gutil.makecrs(crs).to_proj4())
    box = gutil.box2shp(shape, crs, mat)[0]
    for i in idx.index:
        if box.intersects(idx.loc[i]['geometry']):
            print(chan)
            raster = gio.read_raster(idx.loc[i]['path'], chan)
            if isinstance(out, tuple):
                out = gnp.frombox(*out, len(chan), raster.dtype)
            print('merge ...', idx.loc[i]['path'], end=' ')
            match_one(raster, out, chan, step, order)
            print('end')
    return out
    
if __name__ == '__main__':
    wkt = 'PROJCS["China_Lambert_Conformal_Conic",GEOGCS["GCS_Beijing_1954",DATUM["Beijing_1954",SPHEROID["Krassowsky_1940",6378245,298.3,AUTHORITY["EPSG","7024"]],AUTHORITY["EPSG","6214"]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433]],PROJECTION["Lambert_Conformal_Conic_2SP"],PARAMETER["standard_parallel_1",30],PARAMETER["standard_parallel_2",62],PARAMETER["latitude_of_origin",0],PARAMETER["central_meridian",105],PARAMETER["false_easting",0],PARAMETER["false_northing",0],UNIT["metre",1,AUTHORITY["EPSG","9001"]]]'
    wktesri = open('全国县界行政区划2010_project.prj').read()
    from pygis.draw import make_paper
    idx = build_index('../data/class')
    
    china = read_shp('./全国县界行政区划2010_project.shp')
    paper = make_paper(china[china['Province']=='河南省'], (1024,768))
    paper = (paper[0], wkt, paper[2])
    match_idx(paper, idx)



from shapely.geometry import GeometryCollection
from shapely.geometry import Polygon
import geopandas as gpd
import pyproj, numpy as np

def makecrs(prj): return pyproj.CRS(prj)

def shp2box(shape, scale, margin=0.05):
    if isinstance(shape, gpd.GeoDataFrame):
        shape = shape['geometry']
    shapes = shape.values
    kmargin = margin/(1-2*margin)
    geoms = list(shape.values)
    bounds = GeometryCollection(geoms).bounds
    l,t,r,b = bounds
    w,h = r-l, b-t
    if isinstance(scale, tuple):
        ox, oy = (l+r)/2, (t+b)/2
        W, H = np.array(scale) * (1-margin*2)
        scale = max(w/W, h/H)
        if w/h > W/H: h = w/W*H
        else: w = h/H*W
        l, t, r, b = (ox-w/2, oy-h/2, ox+w/2, oy+h/2)
    offsetx, offsety = l-w*kmargin, b+h*kmargin
    shp = np.array((h,w))*(1+(kmargin*2))/scale

    shp = (tuple(shp.round().astype(np.int)))
    m = np.array([offsetx, scale, 0, offsety, 0, -scale]).reshape((2,3))
    return (shp, shape.crs, m)

def box2shp(shape, crs, m):
    hs, ws = shape[:2]
    xx = np.linspace(0,ws,100).reshape((-1,1))*[1,0]
    yy = np.linspace(0,hs,100).reshape((-1,1))*[0,1]
    xy = np.vstack((xx, yy+[ws,0], xx[::-1]+[0,hs],yy[::-1]))
    xy = np.dot(m[:,1:], xy.T) + m[:,:1]
    return gpd.GeoSeries([Polygon(xy.T)], crs=pyproj.CRS(crs).to_proj4())

def shp_bounds(shape):
    bds = shape.bounds
    return [bds['minx'].min(), bds['miny'].min(), 
            bds['maxx'].max(), bds['maxy'].max()]

def box_boundx(shape, crs, m):
    return shp_bounds(box2shp(shape, crs, m))

if __name__ == '__main__':
    shp = read_shp('../../tasks/country_china_wheat_2018/region.shp')
    from time import time
    start = time()
    raster = shp2raster(shp, 250, 0, style='lab')
    print(time()-start)
    plt.imshow(raster.imgs[0])
    plt.show()


import geopandas as pd
from .. import io as gio
from .. import util as gutil
from .. import geoarray

import numpy as np
from shapely.geometry import Polygon, GeometryCollection
from imageio import imread, imsave
from math import ceil, floor

max_lon = 85.0511287798
max_dis = 20037508.343

def build_grid_idx(level=0, box=None):
    n = 2**level

    ls = np.mgrid[0:n]

    minlat, minlon, maxlat, maxlon = box.bounds
    # print(minlat, minlon, maxlat, maxlon)
    step = max_dis * 2 / n

    sr = floor((minlat+max_dis)/step)
    er = ceil((maxlat+max_dis)/step)

    sc = floor((max_dis-maxlon)/step)
    ec = ceil((max_dis-minlon)/step)

    rcs = np.mgrid[sr:er, sc:ec].reshape(2,-1)

    # rgs = np.mgrid[-max_dis+sr*step:-max_dis+er*step:(n+1)*1j]

    rgs = np.mgrid[floor(minlat/step)*step:ceil(maxlat/step)*step:(er-sr+1)*1j,
                   ceil(maxlon/step)*step:floor(minlon/step)*step:(ec-sc+1)*1j]

    print(sr, er, sc, ec)
    p1, p2 = rgs[:,:-1,:-1], rgs[:,:-1,1:]
    p3, p4 = rgs[:,1:,1:], rgs[:,1:,:-1]
    pts = np.array([p1, p2, p3, p4])
    plgs = pts.reshape(4,2,-1).transpose(2,0,1)

    plgs = [Polygon(i) for i in plgs]
    print(len(plgs), rcs.shape)
    data = {'geometry':plgs, 'x':rcs[0], 'y':rcs[1]}
    zs = np.ones(len(plgs), dtype=np.uint8)
    data['z'] = zs * level
    return pd.GeoDataFrame(data, crs=3857)

def build_geo_tif(img, crs, box):
    h, w = img.shape[:2]
    mat = [[box[0], (box[2]-box[0])/(w-1), 0],
           [box[3], 0, -(box[3]-box[1])/(h-1)]]
    return geoarray(img, crs=crs, mat=np.array(mat))

def build_online_tiles(level=3, box=None):
    if not box is None:
        shp = gutil.box2shp(*box).to_crs(3857)[0]
    if level is None and not box is None:
        a,b,c,d = shp.bounds
        l = ((c-a)**2 + (d-b)**2)**0.5
        s = (box[0][0]**2 + box[0][1]**2)**0.5
        rg = max_dis*2 / 256 / 2 ** np.arange(13)
        k = max(l/s, rg[-1]+1e-3)
        level = np.argmax(rg <= k)
        print(level)
    ts = build_grid_idx(level, shp)
    if not box is None: ts = ts[ts.intersects(shp)]
    return ts

urls = {'arcgis blue':'https://map.geoq.cn/ArcGIS/rest/services/ChinaOnlineCommunity/MapServer/tile/{z}/{y}/{x}',
        'arcgis gray':'https://map.geoq.cn/ArcGIS/rest/services/ChinaOnlineStreetGray/MapServer/tile/{z}/{y}/{x}',
        'arcgis dark':'https://map.geoq.cn/ArcGIS/rest/services/ChinaOnlineStreetPurplishBlue/MapServer/tile/{z}/{y}/{x}',
        'gaode':'https://webst01.is.autonavi.com/appmaptile?style=6&x={x}&y={y}&z={z}',
        'google':'http://mt1.google.com/vt/lyrs=s&x={x}&y={y}&z={z}'}

def download_online_tiles(tiles, path, src='gaode'):
    for i,g,x,y,z in tiles[['geometry','x','y','z']].to_records():
        img = imread(urls[src].format(x=x, y=y, z=z))
        img = build_geo_tif(img, 3857, g.bounds)
        gio.write_tif(img, '%s/%ld_%d_%d.tif'%(path,z,y,x))

if __name__ == '__main__':
    import geonumpy.draw as gdraw
    import geonumpy.match as gmt
    import geonumpy.io as gio

    from glob import glob

    box = gio.read_tif_box('./3.背景TIF.tif')
    buf = gnp.frombox(*box[:3], 3)

    tiles = build_online_tiles(level=7, box=box[:3])
    download_online_tiles(tiles, './tiles', 'gaode')

    idx = gmt.build_index(glob('./tiles/*.tif'))
    rst = gmt.match_idx(idx, box[:3])
    gio.write_tif(rst, 'b.tif')




'''
from modis_download import search as modis_search
from modis_download import download as modis_download
from landsat_search import search as landsat_search
from landsat_download import download as landsat_download
'''


import requests, json, time
from bs4 import BeautifulSoup
client = requests.session()

url_rc = 'https://earthexplorer.usgs.gov/ajax/wrstolatlng?path=%.3d&row=%.3d&type=WRS2&wrsType=Point'
url_set = 'https://earthexplorer.usgs.gov/tabs/save'
url_index = 'https://earthexplorer.usgs.gov/result/index'

data_set_date = '''
    {"tab":1,"destination":2,"coordinates":[
    {"c":"0","a":%.4f,"o":%.4f}],
    "format":"dms","dStart":"%s","dEnd":"%s",
    "searchType":"Std","num":"1000",
    "includeUnknownCC":"1","maxCC":100,"minCC":0,
    "months":%s,"pType":"polygon"}'''

data_set_L8 = '''
    {"tab":3,"destination":4,"criteria":{"12864":{
    "select_20522_5":["%s"],"select_20515_5":["%s"],
    "select_20510_4":[""],"select_20517_4":[""],
    "select_20518_4":[""],"select_20513_3":[""],
    "select_20519_3":[""]}},"selected":"12864"}'''

data_set_L7 = '''
    {"tab":3,"destination":4,"criteria":{"12267":{
    "select_19893_5":["%s"],"select_19883_5":["%s"],
    "select_19890_3":[""],"select_19886_4":[""],
    "select_19892_3":[""],"select_19885_3":[""]}},"selected":"12267"}'''

data_set_L45 = '''
    {"tab":3,"destination":4,"criteria":{"12266":{
    "select_19881_5":["%s"],"select_19874_5":["%s"],
    "select_19880_3":[""],"select_19876_4":[""],
    "select_19877_3":[""],"select_25173_3":[""],
    "select_19875_3":[""]}},"selected":"12266"}'''

def get_latlon_byrc(path, row):
    rst = client.get(url_rc%(path, row))
    return rst.json()['coordinates'][0]

def set_rc_date(coord, start, end, month=None):
    if month is None: month = ['']+list(range(12))
    month = json.dumps(["%s"%i for i in month])
    data = data_set_date%(coord[0], coord[1], start, end, month)
    data = data.replace(' ', '').replace('\n', '')
    rst = client.post(url_set, {'data':data})
    if rst.content != b'1': print('Error:set rc data')

def set_criterial(product, landcloud='', scenecloud=''):
    datastr = {8:data_set_L8, 7:data_set_L7, 45:data_set_L45}[product]
    data = datastr%(landcloud, scenecloud)
    data = data.replace(' ', '').replace('\n', '')
    rst = client.post(url_set, {'data':data})
    if rst.content != b'1': print('Error:set criterial data')

def get_content(product, page):
    cid = {7:12267, 8:12864, 45:12266}[product]
    rst = client.post(url_index, {'collectionId':cid, 'pageNum':page})
    return rst.content.decode('utf-8')

def parse_record(cont):
    soup = BeautifulSoup(cont, 'html.parser')
    records = soup.findAll(attrs={'class':'metadata', 'data-url':True})
    return [i.text for i in records[1::3]]

def search(product, prs, start, end, month=None, lcloud='', scloud='', page=1):
    records = []
    for path, row in prs:
        client.cookies.clear()
        print('searching %.3d %.3d, ...'%(path, row))
        coord = get_latlon_byrc(path, row)
        set_rc_date(coord, start, end, month)
        set_criterial(product, lcloud, scloud)
        for i in range(page):
            cont = get_content(product, i+1)
            rds = parse_record(cont)
            print(len(rds))
            records.extend(rds)
            if len(rds)<10: break
    return records

def to_order(records):
    f = open('order.txt', 'w')
    f.write('\n'.join(records))
    f.close()

# 这里输入需要下载的行列号
blocks = [(123,43), (119,37), (122,37)]

if __name__ == '__main__':
    # 等级（7，8），行列号，起始，结束日期，月份限定，陆地云量，传感器云量，页数
    records = search(7, blocks, '06/30/2018', '07/30/2019', None, 10, 10, 1)
    for i in records: print(i) # 打印
    #to_order(records) # 存储为订单文件




import geonumpy.io as gio
import matplotlib.pyplot as plt


if __name__ == '__main__':
    gdf = gio.read_shp('../data/shape/shandong.shp')
    print(gdf)
    gdf.plot()
    plt.show()
    
    print('read modis hdf data:')
    modis = gio.read_hdf('../data/modis/MOD09Q1.A2019017.h28v05.006.2019030120612.hdf', 0)
    print(modis.shape, '\n', modis.crs, '\n', modis.mat)
    plt.imshow(modis, cmap='gray')
    plt.show()

    print('\nread landsat tif data:')
    landsat = gio.read_tif('../data/landsat/LC08_L1TP_122033_20190506_20190506_01_RT_B5.TIF')
    print(modis.shape, '\n', modis.crs, '\n', modis.mat)
    plt.imshow(landsat, cmap='gray')
    plt.show()

    print('\nsave modis as tif')
    gio.write_tif(modis, '../data/modis/MOD09Q1.A2019017.h25v05.006.2019030120448.tif')

    box = gio.read_tif_box('../data/landsat/LC08_L1TP_122033_20190506_20190506_01_RT_B5.TIF')
    print(box)



import numpy as np

x_pi = np.pi * 3000.0 / 180.0
pi = 3.1415926535897932384626  # π
a = 6378245.0  # 长半轴
ee = 0.00669342162296594323  # 扁率

def wgs84togcj02(lnglat):
    lng, lat = lnglat.T[:2]
    if out_of_china(lng.mean(), lat.mean()):  # 判断是否在国内
        return lnglat
    dlat = transformlat(lng - 105.0, lat - 35.0)
    dlng = transformlng(lng - 105.0, lat - 35.0)
    radlat = lat / 180.0 * pi
    magic = np.sin(radlat)
    magic = 1 - ee * magic * magic
    sqrtmagic = np.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * pi)
    dlng = (dlng * 180.0) / (a / sqrtmagic * np.cos(radlat) * pi)
    mglat = lat + dlat
    mglng = lng + dlng
    return np.array([mglng, mglat]).T


def gcj02towgs84(lnglat):
    lng, lat = lnglat.T[:2]
    if out_of_china(lng.mean(), lat.mean()): return lnglat
    dlat = transformlat(lng - 105.0, lat - 35.0)
    dlng = transformlng(lng - 105.0, lat - 35.0)
    radlat = lat / 180.0 * pi
    magic = np.sin(radlat)
    magic = 1 - ee * magic * magic
    sqrtmagic = np.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * pi)
    dlng = (dlng * 180.0) / (a / sqrtmagic * np.cos(radlat) * pi)
    mglat = lat + dlat
    mglng = lng + dlng
    return np.array([lng * 2 - mglng, lat * 2 - mglat]).T


def transformlat(lng, lat):
    ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + \
        0.1 * lng * lat + 0.2 * np.sqrt(np.abs(lng))
    ret += (20.0 * np.sin(6.0 * lng * pi) + 20.0 *
            np.sin(2.0 * lng * pi)) * 2.0 / 3.0
    ret += (20.0 * np.sin(lat * pi) + 40.0 *
            np.sin(lat / 3.0 * pi)) * 2.0 / 3.0
    ret += (160.0 * np.sin(lat / 12.0 * pi) + 320 *
            np.sin(lat * pi / 30.0)) * 2.0 / 3.0
    return ret


def transformlng(lng, lat):
    ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + \
        0.1 * lng * lat + 0.1 * np.sqrt(np.abs(lng))
    ret += (20.0 * np.sin(6.0 * lng * pi) + 20.0 *
            np.sin(2.0 * lng * pi)) * 2.0 / 3.0
    ret += (20.0 * np.sin(lng * pi) + 40.0 *
            np.sin(lng / 3.0 * pi)) * 2.0 / 3.0
    ret += (150.0 * np.sin(lng / 12.0 * pi) + 300.0 *
            np.sin(lng / 30.0 * pi)) * 2.0 / 3.0
    return ret

def out_of_china(lng, lat):
    msk = (lng > 72.004) & (lng < 137.8347)
    msk &= (lat > 0.8293) & (lat < 55.8271)
    return not msk

def lat2y(lat):
    rad = np.clip(lat, -85.0511287798, 85.0511287798) * np.pi / 180
    return np.log(np.tan(np.pi / 4 + rad / 2)) / np.pi * 180

def y2lat(y):
    return (np.arctan(np.exp(y / 180 * np.pi)) - np.pi / 4) * 2 / np.pi * 180

def gaode_adjust(xy):
    return np.array([xy[:,0], lat2y(xy[:,1])]).T

def gaode_adjust_(xy):
    return np.array([xy[:,0], y2lat(xy[:,1])]).T

def wgs2gaode(xy):
    return gaode_adjust(wgs84togcj02(xy))

def gaode2wgs(xy):
    return gcj02towgs84(gaode_adjust_(xy))

if __name__ == '__main__':
    lng, lat = 120, 35
    lnglat = np.array([[120, 35],[120,35]])
    rst = wgs84togcj02(lnglat)



import geonumpy as gnp
import geonumpy.io as gio
import geonumpy.util as gutil
import geonumpy.draw as gdraw
import geonumpy.match as gmt
import numpy as np
import scipy.ndimage as ndimg
import matplotlib.pyplot as plt
from PIL import Image
from glob import glob

def match_class(df):
    shandong = df.to_crs(3857)
    box = gutil.shp2box(shandong, (3600, 2400), 0.15)
    paper = gnp.frombox(*box, dtype=np.uint8)
    idx = gmt.build_index(glob('../data/class/*.tif'))
    gmt.match_idx(idx, out=paper, order=0)
    gio.write_tif(paper, '../data/result/shandong_class.tif')

def city_label(df):
    shandong = df.to_crs(3857)
    box = gutil.shp2box(shandong, (3600, 2400), 0.15)
    paper = gnp.frombox(*box, dtype=np.uint8)
    gdraw.draw_polygon(paper, shandong, np.arange(len(shandong))+1, 0)
    gio.write_tif(paper, '../data/result/shandong_label.tif')

def show_class_label(cls, lab):
    ax1, ax2 = plt.subplot(121), plt.subplot(122)
    ax1.imshow(cls)
    ax2.imshow(lab)
    ax1.set_title('shandong class')
    ax2.set_title('shandong label')
    plt.show()

def statistic(cls, lab, df):
    forest = ndimg.sum(cls==2, lab, np.arange(lab.max())+1)
    total = np.bincount(lab.ravel())[1:]
    df['area'] = total
    df['forest'] = forest
    df['ratio'] = forest/total
    return df

def draw_ratio(cls, lab, df):
    shandong = df.to_crs(3857)
    paper = cls.copy()
    paper[lab==0] = 12
    
    lut = np.array([[0  ,0  ,0  ],
                    [168,168,0  ],
                    [20 ,119,73 ],
                    [169,208,95 ],
                    [56 ,168,0  ],
                    [126,206,244],
                    [0  ,86 ,154],
                    [112,168,0  ],
                    [147,47 ,20 ],
                    [202,202,202],
                    [0  ,255,197],
                    [20 ,119,73 ],
                    [255,255,255]], dtype=np.uint8)

    body = [('图例', 'simhei', 72),
            ('rect', 1,  '农田'),
            ('rect', 2,  '森林'),
            ('rect', 3,  '草地'),
            ('rect', 4,  '灌丛'),
            ('rect', 5,  '湿地'),
            ('rect', 6,  '水体'),
            ('rect', 7,  '苔原'),
            ('rect', 8,  '隔水层'),
            ('rect', 9,  '裸地'),
            ('rect', 10, '冰雪')]
    
    lut[1:-2] = lut[1:-2] * 0.6 + 255 * 0.4

    shandong['lab'] = shandong['ratio'].apply(lambda x:'%.2f%%'%(x*100))
    gdraw.draw_polygon(paper, shandong, 0, 2)
    gdraw.draw_polygon(paper, shandong[shandong['ratio']>0.1], 11, 8)
    gdraw.draw_lab(paper, shandong, 'lab', 0, ('simhei', 48), 'ct')
    # 底图，位置，内容，空隙，矩形尺寸及线宽，字体字号颜色，外边框宽度
    gdraw.draw_style(paper, 60, -60, body, mar=(20, 30),
        recsize=(120,60,0), font=('simsun', 60, 0), color=0, box=0)
    
    gdraw.draw_unit(paper, -120, -60, 0.3, 30, ('times', 48), 0, 'km', 3, anc='r')
    gdraw.draw_text(paper, '山东省森林覆盖率统计', 80, 60, 0, ('simkai', 128))
    gdraw.draw_N(paper, -240, 240, ('simhei', 100), 2, 100, 0)
    gdraw.draw_bound(paper, 5, 5, -5, -5, 0, 2, clear=None)
    return paper.lookup(lut)

if __name__ == '__main__':
    shandong = gio.read_shp('../data/shape/shandong.shp')
    match_class(shandong)
    city_label(shandong)
    
    cls = gio.read_tif('../data/result/shandong_class.tif')
    lab = gio.read_tif('../data/result/shandong_label.tif')

    tab = statistic(cls, lab, shandong)
    print(tab[['name', 'area', 'forest', 'ratio']])

    rst = draw_ratio(cls, lab, shandong)

    Image.fromarray(rst).show()
    Image.fromarray(rst).save('../data/result/shandong_forest.png')



import numpy as np
from numpy.linalg import inv

class GeoArray(np.ndarray):
    def __new__(cls, input_array, crs=4326, mat=None):
        obj = np.asarray(input_array).view(cls)
        obj.crs, obj.mat = crs, mat.reshape((2,3))
        return obj
    
    def __array_finalize__(self, obj):
        if obj is None: return
        self.crs = getattr(obj, 'crs', '')
        self.mat = getattr(obj, 'mat', None)
    
    def __getitem__(self, item):
        sliced = True & isinstance(item, tuple)
        if sliced:
            sliced &= len(item) in (2, 3)
            sliced &= isinstance(item[1], slice)
            sliced &= isinstance(item[0], slice)
        def gs(s): return (s.start or 0, s.step or 1)
        if sliced:
            (s1,d1), (s2,d2) = gs(item[0]), gs(item[1])
            obj = super(GeoArray, self).__getitem__(item)
            if not obj.mat is None:
                m, offset = obj.mat[:,1:], obj.mat[:,:1]
                o = np.dot(m, [[s2],[s1]]) + offset
                t = m * [d2, d1]
                obj.mat = np.hstack((o,t))
            return obj
        if not sliced:
            return super().__getitem__(item).__array__()

    def __array_wrap__(self, out_arr, context=None):
        if out_arr.shape[:2] != self.shape[:2]:
            out_arr = out_arr.__array__()
        return out_arr

    @property
    def imat(self):
        imat = np.vstack((self.mat[:,[1,2,0]], [[0,0,1]]))
        return np.linalg.inv(imat)[:2,[2,0,1]]

    @property
    def imat1(self): return self.imat.ravel()[[1,2,4,5,0,3]]

    def project(self, x, y):
        x += 0.5; y += 0.5
        m, offset = self.mat[:,1:], self.mat[:,:1]
        xy = np.array([x, y]).reshape((2,-1))
        return np.dot(m, xy) + offset

    def invpro(self, e, n):
        m, offset = self.mat[:,1:], self.mat[:,:1]
        en = np.array([e, n]).reshape((2,-1))
        return np.dot(inv(m), en - offset) - 0.5

    def channels(self, n=None):
        if n is None:
            return 1 if self.ndim==2 else self.shape[2]
        else:
            return self if self.ndim==2 else self[:,:,n]

    def lookup(self, lut):
        return GeoArray(lut[self], self.crs, self.mat)

    def getbox(self): 
        return (self.shape[:2], self.crs, self.mat)


def frombox(shp, crs, mat, chan=1, dtype=np.uint8):
    if chan>1: shp += (chan,)
    return GeoArray(np.zeros(shp, dtype=dtype), crs, mat)

def geoarray(arr, crs=None, mat=np.array([[1,1,0],[1,0,1]])):
    return GeoArray(arr, crs, mat)

        
if __name__ == '__main__':
    prj = np.array([0,1,0, 0,0,1])
    a = GeoArray(np.ones((5,5)), crs=4326, mat=prj)
    print(a.crs)
    print(a.mat)
    b = a+1
    print(a.crs)
    print(a.mat)
    c = a[1::2,1::2]
    print(c.crs)
    print(c.mat)



import geonumpy as gnp
import numpy as np

if __name__ == '__main__':
    mat = np.array([[0,1,0],[0,0,1]])
    garr = gnp.geoarray(np.ones((5,5)), crs=4326, mat=mat)
    print(garr.crs)
    print(garr.mat)
    
    garr2 = garr+1
    print(garr2.crs)
    print(garr2.mat)
    
    garr3 = garr[1::2,1::2]
    print(garr3.crs)
    print(garr3.mat)

    box = garr.getbox()
    print(box)

    garr4 = gnp.frombox(*box, dtype=np.uint8)
    print(garr4.getbox())

    garr_mc = gnp.geoarray(np.ones((5,5,3)), crs=4326, mat=mat)
    print(garr_mc.crs)
    print(garr_mc.mat)
    



import geonumpy.io as gio
import numpy as np
import geonumpy as gnp
import geonumpy.match as gmt
from glob import glob
import geonumpy.util as gutil
import geonumpy.draw as gdraw
import matplotlib.pyplot as plt
import geopandas as gpd

def all_source():
    fs = glob('../data/modis/*.hdf')
    ax1 = plt.subplot(131)
    ax1.imshow(gio.read_hdf(fs[0], 0))
    ax2 = plt.subplot(132)
    ax2.imshow(gio.read_hdf(fs[1], 0))
    ax3 = plt.subplot(133)
    ax3.imshow(gio.read_hdf(fs[2], 0))
    plt.show()
    
def match_one_test():
    shandong = gio.read_shp('../data/shape/shandong.shp')
    shandong = shandong.to_crs(3857)
    info = gutil.shp2box(shandong, (2048,1536), 0.05)
    #paper = gnp.frombox(*info, dtype=np.int16)
    path = '../data/modis/MOD09Q1.A2019017.h27v05.006.2019030120430.hdf'
    raster = gio.read_hdf(path)
    print(raster.shape)
    rst = gmt.match_one(raster, info, 0)
    print(rst.shape)
    plt.imshow(rst)
    plt.show()

def match_multi_test():
    shandong = gio.read_shp('../data/shape/shandong.shp')
    shandong = shandong.to_crs(3857)
    info = gutil.shp2box(shandong, (2048,1536), 0.05)
    fs = glob('../data/modis/*.hdf')
    rasters = [gio.read_hdf(i, 0) for i in fs]
    rst = gmt.match_multi(rasters, info, 0)
    plt.imshow(rst)
    plt.show()

def build_idx_test():
    fs = glob('../data/modis/*.hdf')
    idx = gmt.build_index(fs)
    idx.plot()
    plt.show()

def match_idx_test():
    shandong = gio.read_shp('../data/shape/shandong.shp')
    shandong = shandong.to_crs(3857)
    info = gutil.shp2box(shandong, (2048,768*2), 0.05)
    
    fs = glob('../data/modis/*.hdf')
    idx = gmt.build_index(fs)
    
    rst = gmt.match_idx(idx, info, chan=[0])
    plt.imshow(rst)
    plt.show()

def crs_trans_test():
    path = '../data/modis/MOD09Q1.A2019017.h27v05.006.2019030120430.hdf'
    raster = gio.read_hdf(path, 0)
    outshp = gutil.box2shp(*raster.getbox()).to_crs(3857)
    box = gutil.shp2box(outshp, 1000, 0)
    paper = gnp.frombox(*box, dtype=np.int16)
    gmt.match_one(raster, out=paper)

    plt.imshow(paper)
    plt.show()
    
if __name__ == '__main__':
    all_source()
    match_one_test()
    match_multi_test()
    
    build_idx_test()
    
    match_idx_test()
    
    crs_trans_test()



import geonumpy as gnp
import geonumpy.io as gio
import geonumpy.util as gutil
import geonumpy.draw as gdraw
import geonumpy.match as gmt
import numpy as np
import matplotlib.pyplot as plt
from PIL import Image
from glob import glob
    
def draw_simple():
    shandong = gio.read_shp('../data/shape/shandong.shp')
    shandong = shandong.to_crs(3857)
    box = gutil.shp2box(shandong, (3600, 2400), 0.1)
    paper = gnp.frombox(*box, chan=1, dtype=np.uint8)
    paper[:] = 255
    gdraw.draw_polygon(paper, shandong, 0, 2)
    gdraw.draw_ruler(paper, 80, 50, -80, -50, 1, 4326, ('times', 32), 0, 2, 5)
    gdraw.draw_lab(paper, shandong, 'name', 0, ('simhei', 32), 'ct')
    gdraw.draw_unit(paper, -180, -100, 0.3, 30, ('times', 48), 0, 'km', 3, anc='r')
    gdraw.draw_text(paper, '山东省', 180, 120, 0, ('simkai', 128))
    gdraw.draw_N(paper, -240, 240, ('simhei', 100), 2, 100, 0)
    return paper

def draw_style():
    paper = gnp.geoarray(np.zeros((480,1024), dtype=np.uint8))

    body = [('Style', 'simhei', 72),
            ('blank',50),
            ('line', 1,  'this is line style'),
            ('circle', 2,  'this is circle style'),
            ('rect', 3,  'this is rect style')]

    lut = np.array([[255,255,255],
                    [255,0  ,0  ],
                    [0  ,255,0  ],
                    [0  ,0  ,255],
                    [0  ,0  ,0  ]], dtype=np.uint8)
    gdraw.draw_style(paper,128,-20, body, mar=(20, 30),
        recsize=(120,60,3), font=('simsun', 60), color=4, box=5)
    return paper.lookup(lut)

def draw_grade():
    shandong = gio.read_shp('../data/shape/shandong.shp')
    shandong = shandong.to_crs(3857)
    box = gutil.shp2box(shandong, (3600, 2400), 0.1)
    paper = gnp.frombox(*box, dtype=np.uint8)
    
    areas = shandong.area
    grade_lut = np.array([3]*60 + [2]*30 + [1]*10, dtype=np.uint8)
    vs = (areas-areas.min())/(areas.max()-areas.min())*99
    grade = grade_lut[vs.astype(int)]
    print(grade)
    
    gdraw.draw_polygon(paper, shandong, grade, 0)
    gdraw.draw_polygon(paper, shandong, 4, 2)
    
    gdraw.draw_ruler(paper, 80, 50, -80, -50, 1, 4326, ('times', 32), 4, 2, 5)
    gdraw.draw_lab(paper, shandong, 'name', 4, ('simhei', 32), 'ct')
    gdraw.draw_unit(paper, -180, -100, 0.3, 30, ('times', 48), 4, 'km', 3, anc='r')
    gdraw.draw_text(paper, '山东省', 180, 120, 4, ('simkai', 128))
    gdraw.draw_N(paper, -240, 240, ('simhei', 100), 2, 100, 4)

    body = [('图例', 'simhei', 72),
            ('rect', 1,  '特大城市'),
            ('rect', 2,  '中型城市'),
            ('rect', 3,  '一般城市')]
    # 底图，位置，内容，空隙，矩形尺寸及线宽，字体字号颜色，外边框宽度
    gdraw.draw_style(paper, 150, -90, body, mar=(20, 30),
        recsize=(120,60,2), font=('simsun', 60, 4), color=4, box=0)
    
    lut = np.array([[255,255,255],
                    [255,200,100],
                    [255,255,128],
                    [255,255,200],
                    [0  ,0  ,0  ]], dtype=np.uint8)
                    
    return paper.lookup(lut)

def draw_class():
    # ===== look up table =====
    lut = np.array([[0  ,0  ,0  ],
                    [168,168,0  ],
                    [20 ,119,73 ],
                    [169,208,95 ],
                    [56 ,168,0  ],
                    [126,206,244],
                    [0  ,86 ,154],
                    [112,168,0  ],
                    [147,47 ,20 ],
                    [202,202,202],
                    [0  ,255,197],
                    [255,255,255]], dtype=np.uint8)

    # ===== read shape file and make a paper =====
    liaoning = gio.read_shp('../data/shape/shandong.shp')
    liaoning = liaoning.to_crs(3857)
    box = gutil.shp2box(liaoning, (3600, 2400), 0.15)
    paper = gnp.frombox(*box, dtype=np.uint8)

    # ===== match the class tif into paper
    fs = glob('../data/class/*.tif')
    idx = gmt.build_index(fs)
    gmt.match_idx(idx, out=paper, order=0)

    msk = paper * 0
    gdraw.draw_polygon(msk, liaoning, 255, 0)
    paper[msk==0] = 11

    body = [('图例', 'simhei', 72),
            ('rect', 1,  '农田'),
            ('rect', 2,  '森林'),
            ('rect', 3,  '草地'),
            ('rect', 4,  '灌丛'),
            ('rect', 5,  '湿地'),
            ('rect', 6,  '水体'),
            ('rect', 7,  '苔原'),
            ('rect', 8,  '隔水层'),
            ('rect', 9,  '裸地'),
            ('rect', 10, '冰雪')]
    # 底图，位置，内容，空隙，矩形尺寸及线宽，字体字号颜色，外边框宽度
    gdraw.draw_style(paper, 60, -60, body, mar=(20, 30),
        recsize=(120,60,0), font=('simsun', 60, 0), color=0, box=0)

    gdraw.draw_unit(paper, -120, -60, 0.3, 30, ('times', 48), 0, 'km', 3, anc='r')

    gdraw.draw_text(paper, '山东省土地利用类型', 80, 60, 0, ('simkai', 128))

    gdraw.draw_N(paper, -240, 240, ('msyh', 100), 2, 100, 0)

    gdraw.draw_polygon(paper, liaoning, 0, 2)
    
    gdraw.draw_bound(paper, 5, 5, -5, -5, 0, 2, clear=None)
        
    return paper.lookup(lut)
    
    
if __name__ == '__main__':
    
    rst = draw_simple()
    rst = draw_style()
    rst = draw_grade()
    rst = draw_class()
    
    Image.fromarray(rst).save('../doc/imgs/00.png')
    Image.fromarray(rst).show()



import geopandas as gpd
from osgeo import gdal, osr, ogr
from osgeo import gdal_array
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import pyproj
from ..base import GeoArray

def read_shp(path, encoding='utf-8'):
    return gpd.read_file(path, encoding=encoding)

def write_shp(shp, path, encoding='utf-8'): 
    shp.to_file(path, encoding)

def read_hdf(path, chans=None):
    ds = gdal.Open(path)
    sds = ds.GetSubDatasets()
    imgs = []
    if chans is None: chans = range(len(sds))
    if isinstance(chans, int): chans = [chans]
    for i in chans:
        ds = gdal.Open(sds[i][0])
        img = ds.ReadAsArray()
        m = ds.GetGeoTransform()
        m = np.array(m).reshape((2,3))
        prj = ds.GetProjection()
        imgs.append(img)
    if len(chans) == 1: imgs = imgs[0]
    else: imgs = np.array(imgs).transpose((1,2,0))
    return GeoArray(imgs, prj, m)

def read_tif(path, chans=None):
    ds = gdal.Open(path)
    if chans is None: chans = range(ds.RasterCount)
    if isinstance(chans, int): chans = [chans]
    prj = ds.GetProjection()
    m = ds.GetGeoTransform()
    m = np.array(m).reshape((2,3))
    imgs = [ds.GetRasterBand(i+1).ReadAsArray() for i in chans]
    if len(chans)==1: imgs = imgs[0]
    else: imgs = np.array(imgs).transpose((1,2,0))
    return GeoArray(imgs, prj, m)

def read_raster(path, chans=None):
    if 'hdf' in path.lower(): return read_hdf(path, chans)
    if 'tif' in path.lower(): return read_tif(path, chans)

def read_tif_box(path):
    ds = gdal.Open(path)
    prj = ds.GetProjection()
    m = ds.GetGeoTransform()
    m = np.array(m).reshape((2,3))
    #chans = ['Channel:%d'%i for i in range(ds.RasterCount)]
    rs = range(ds.RasterCount)
    chans = [ds.GetRasterBand(i+1).GetDescription() for i in rs]
    shape = (ds.RasterYSize, ds.RasterXSize)
    return (shape, prj, m, chans)

def read_hdf_box(path):
    ds = gdal.Open(path)
    sds = ds.GetSubDatasets()
    rs = gdal.Open(sds[0][0])
    prj = rs.GetProjection()
    m = rs.GetGeoTransform()
    m = np.array(m).reshape((2,3))
    chans = [i[1] for i in sds]
    shape = (rs.RasterYSize, rs.RasterXSize)
    return (shape, prj, m, chans)

def read_raster_box(path):
    if 'hdf' in path.lower(): return read_hdf_box(path)
    if 'tif' in path.lower(): return read_tif_box(path)


def write_tif(raster, path, compress=None, nodata=None):
    #if isinstance(raster, tuple): raster = [raster]
    driver = gdal.GetDriverByName("GTiff")
    tps = {np.uint8:gdal.GDT_Byte, np.int16:gdal.GDT_Int16,
           np.int32:gdal.GDT_Int32, np.uint16:gdal.GDT_UInt16,
           np.uint32:gdal.GDT_UInt32, np.float32:gdal.GDT_Float32,
           np.float64:gdal.GDT_Float64}
    options = [] if compress is None else ['COMPRESS=%s'%compress]
    tif = driver.Create(path, raster.shape[1], raster.shape[0],
                        raster.channels(), tps[raster.dtype.type], options=options)
    tif.SetGeoTransform(raster.mat.ravel())
    crs = osr.SpatialReference()
    crs.ImportFromProj4(pyproj.CRS(raster.crs).to_proj4())
    tif.SetProjection(crs.ExportToWkt())
    for i in range(raster.channels()):
        b = tif.GetRasterBand(i+1)
        b.WriteArray(raster.channels(i))
        if not nodata is None: b.SetNoDataValue(nodata)

def array_as_shp(raster, path, field='class'):
    tif = gdal_array.OpenArray(raster)
    tif.SetGeoTransform(raster.mat.ravel())
    crs = osr.SpatialReference()
    crs.ImportFromProj4(pyproj.CRS(raster.crs).to_proj4())
    tif.SetProjection(crs.ExportToWkt())
    for i in range(raster.channels()):
        tif.GetRasterBand(i+1).WriteArray(raster.channels(i))
    band = tif.GetRasterBand(1)
    driver = ogr.GetDriverByName("ESRI Shapefile")
    outDatasource = driver.CreateDataSource(path)
    srs = osr.SpatialReference()
    srs.ImportFromWkt( tif.GetProjectionRef() )
    outLayer = outDatasource.CreateLayer(path, srs)
    newField = ogr.FieldDefn(field, ogr.OFTInteger)
    outLayer.CreateField(newField)
    gdal.Polygonize(band, None, outLayer, 0, [],callback=None)  
    outDatasource.Destroy()

def show_raster(raster, c=0):
    plt.imshow(raster.imgs[c])

def show_shp(shp):
    shp.plot()
    plt.show()
    
if __name__ == '__main__':
    shp = read_shp('../../tasks/country_china_wheat_2018/region.shp')
    #plot_shp(shp)



import geonumpy.io as gio
import geonumpy.pretreat as gpt
import matplotlib.pyplot as plt

def degap():
    img = gio.read_tif('../data/landsat-gap/LE07_L1TP_123037_20180721_20180816_01_T1_sr_ndvi.tif')
    degapimg = gpt.degap(img.copy(), img==-9999, 10)

    ax1 = plt.subplot(121)
    ax1.set_title('ori image')
    ax1.imshow(img, cmap='gray')
    
    ax2 = plt.subplot(122)
    ax2.set_title('degaped image')
    ax2.imshow(degapimg, cmap='gray')
    plt.show()

if __name__ == '__main__':
    degap()



import numpy as np

def ndvi(raster1, raster2):
    b1 = np.clip(raster1[0], 1, 1e8)
    b2 = np.clip(raster2[0], 1, 1e8)
    ndvi = (((b2-b1)/(b2+b1)+1)/2*255+0.5).astype(np.uint8)
    return (ndvi, raster1[1], raster1[2])


import geonumpy.download as gdl

def modis_test():
    imgs = gdl.modis_search(['MOD09Q1', 'MOD11A2'],
                            level=6,
                            areas=['h25v05', 'h27v05', 'h28v05'],
                            terms=[(2019, (0,30))])

    gdl.download(imgs, './download/')

def landsat_test():
    blocks = [(123,43), (119,37), (122,37)]
    # 等级（7，8），行列号，起始，结束日期，月份限定，陆地云量，传感器云量，页数
    records = gdl.landsat_search(7, blocks, '06/30/2018', '07/30/2019', None, 10, 10, 1)
    for i in records: print(i) # 打印



#!/usr/bin/env python3


from setuptools import setup
from setuptools import find_packages
import setuptools

setup(
    name="geonumpy",
    version="0.10",
    author="YXDragon",
    author_email="yxdragon@imagepy.org",
    license = "BSD 3-Clause",
    url = "http://imagepy.org/",
    description = "combine geo crs and mat with numpy array",
    install_requires = [
        "scikit-image",
        "numpy",
        "scipy",
        "matplotlib",
        "pandas",
        "geopandas",
    ],
    packages=setuptools.find_packages(),
)
